import _extends from "@babel/runtime/helpers/esm/extends";
import { useReducer, useRef } from 'react';
import Domain from '../domain/index.js';
import useDomainEventSubscriptions from './useDomainEventSubscriptions.js';
var domain = new Domain();
var ACTIONS = {
  SET_STATE: 'SET_STATE',
  TOOGLE_TAB: 'toggleTab'
};
var initialState = {
  tasks: [],
  toggleTab: false
};
var reducer = function reducer(state, action) {
  switch (action.type) {
    case ACTIONS.SET_STATE:
      return _extends({}, state, action.payload);
    case ACTIONS.TOOGLE_TAB:
      return _extends({}, state, {
        toggleTab: !state.toggleTab
      });
    default:
      return state;
  }
};

// Create a custom hook that uses the reducer and returns the state and dispatch function
var useState = function useState() {
  var _useReducer = useReducer(reducer, _extends({}, initialState)),
    state = _useReducer[0],
    dispatch = _useReducer[1];
  var stateRef = useRef(state);
  stateRef.current = state;
  var getState = function getState() {
    return stateRef.current;
  };
  var setState = function setState(state) {
    return dispatch({
      type: ACTIONS.SET_STATE,
      payload: state
    });
  };
  var toggleTab = function toggleTab() {
    return dispatch({
      type: ACTIONS.TOOGLE_TAB
    });
  };
  var executeUseCase = function executeUseCase(useCaseName, params) {
    return domain.get(useCaseName).execute(params).then(function (result) {
      return setState(result);
    });
  };
  useDomainEventSubscriptions(domain, executeUseCase);
  var getTask = function getTask(taskId) {
    return getState().tasks.find(function (task) {
      return task.id === taskId;
    });
  };
  var runSimpleTask = function runSimpleTask(task) {
    return executeUseCase('run_simple_task_use_case', _extends({
      localState: stateRef.current
    }, task));
  };
  var runTask = function runTask(task) {
    return executeUseCase('run_task_use_case', _extends({
      localState: stateRef.current
    }, task));
  };
  var setPercentage = function setPercentage(taskId, workId, percentage) {
    return executeUseCase('set_work_percentage_task_use_case', {
      localState: stateRef.current,
      taskId: taskId,
      workId: workId,
      percentage: percentage
    });
  };
  var finishWork = function finishWork(taskId, workId, result) {
    if (result === void 0) {
      result = null;
    }
    return executeUseCase('finish_work_task_use_case', {
      localState: stateRef.current,
      taskId: taskId,
      workId: workId,
      result: result
    });
  };
  var cancelWork = function cancelWork(taskId, workId) {
    return executeUseCase('cancel_work_task_use_case', {
      localState: stateRef.current,
      taskId: taskId,
      workId: workId
    });
  };
  var errorWork = function errorWork(taskId, workId, log) {
    return executeUseCase('error_work_task_use_case', {
      localState: stateRef.current,
      log: log,
      taskId: taskId,
      workId: workId
    });
  };
  return {
    cancelWork: cancelWork,
    errorWork: errorWork,
    finishWork: finishWork,
    getState: getState,
    getTask: getTask,
    runSimpleTask: runSimpleTask,
    runTask: runTask,
    setPercentage: setPercentage,
    toggleTab: toggleTab
  };
};
export default useState;